@@ -1,5 +1,5 @@
 test_that("read_resource() returns a tibble", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   df <- data.frame("col_1" = c(1, 2), "col_2" = c("a", "b"))
   p <- add_resource(p, "new", df)
@@ -9,29 +9,120 @@ test_that("read_resource() returns a tibble", {
   expect_s3_class(read_resource(p, "new"), "tbl")         # via df
 })
 
-test_that("read_resource() returns error on incorrect Data Package", {
-  expect_error(
-    read_resource(list(), "deployments"),
-    paste(
-      "`package` must be a list describing a Data Package,",
-      "created with `read_package()` or `create_package()`."
+test_that("read_resource() allows column selection", {
+  skip_if_offline()
+  p <- example_package
+
+  # Single column
+  expect_named(
+    read_resource(p, "deployments", col_select = "start"),
+    "start"
+  )
+  expect_identical(
+    read_resource(p, "deployments", col_select = "start"),
+    dplyr::select(
+      read_resource(p, "deployments"),
+      "start"
     ),
+    ignore_attr = "spec" # read_delim() returns vroom::spec(), select() does not
+  )
+
+  # Multiple columns
+  expect_named(
+    read_resource(p, "deployments", col_select = c("deployment_id", "start")),
+    c("deployment_id", "start")
+  )
+  expect_identical(
+    read_resource(p, "deployments", col_select = c("deployment_id", "start")),
+    dplyr::select(
+      read_resource(p, "deployments"),
+      "deployment_id",
+      "start"
+    ),
+    ignore_attr = "spec"
+  )
+
+  # Different order
+  expect_named(
+    read_resource(
+      p,
+      "deployments",
+      col_select = c("start", "deployment_id", "comments")
+    ),
+    c("start", "deployment_id", "comments"),
+    ignore.order = FALSE
+  )
+  expect_identical(
+    read_resource(
+      p,
+      "deployments",
+      col_select = c("start", "deployment_id", "comments")
+    ),
+    dplyr::select(
+      read_resource(p, "deployments"),
+      "start",
+      "deployment_id",
+      "comments"
+    ),
+    ignore_attr = "spec"
+  )
+})
+
+test_that("read_resource() returns error on column selection not in schema", {
+  skip_if_offline()
+  p <- example_package
+
+  # One column
+  expect_error(
+    read_resource(p, "deployments", col_select = "no_such_column"),
+    class = "frictionless_error_colselect_mismatch"
+  )
+  expect_error(
+    read_resource(p, "deployments", col_select = "no_such_column"),
+    regexp = "Can't find column \"no_such_column\" in field names.",
+    fixed = TRUE
+  )
+  expect_error(
+    read_resource(p, "deployments", col_select = "no_such_column"),
+    regexp = "Field names: \"deployment_id\", \"longitude\", \"latitude\", \"start\", and \"comments\".",
+    fixed = TRUE
+  )
+
+  # Partial match, multiple columns
+  expect_error(
+    read_resource(
+      p,
+      "deployments",
+      col_select = c("no_such_column", "start", "no_such_column_either")
+    ),
+    class = "frictionless_error_colselect_mismatch"
+  )
+  expect_error(
+    read_resource(
+      p,
+      "deployments",
+      col_select = c("no_such_column", "start", "no_such_column_either")
+    ),
+    regexp = "Can't find columns \"no_such_column\" and \"no_such_column_either\" in field names.",
     fixed = TRUE
   )
 })
 
-test_that("read_resource() returns error on incorrect resource", {
-  testthat::skip_if_offline()
+test_that("read_resource() returns error on invalid Data Package", {
+  expect_error(
+    read_resource(list(), "deployments"),
+    class = "frictionless_error_package_invalid"
+  )
+})
+
+test_that("read_resource() returns error on invalid resource", {
+  skip_if_offline()
   p <- example_package
 
   # No such resource
   expect_error(
     read_resource(p, "no_such_resource"),
-    paste(
-      "Can't find resource `no_such_resource` in `deployments`,",
-      "`observations`, `media`."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_resource_not_found"
   )
 
   # Create invalid package and add properties one by one to pass errors
@@ -41,7 +132,11 @@ test_that("read_resource() returns error on incorrect resource", {
   # No path or data
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Resource `deployments` must have property `path` or `data`.",
+    class = "frictionless_error_resource_without_path_data"
+  )
+  expect_error(
+    read_resource(p_invalid, "deployments"),
+    regexp = "Resource \"deployments\" must have a path or data property.",
     fixed = TRUE
   )
 
@@ -49,46 +144,35 @@ test_that("read_resource() returns error on incorrect resource", {
   p_invalid$resources[[1]]$path <- "http://example.com/no_such_file.csv"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Can't find file at `http://example.com/no_such_file.csv`.",
-    fixed = TRUE
+    class = "frictionless_error_url_not_found"
   )
 
   # No file at path
   p_invalid$resources[[1]]$path <- "no_such_file.csv"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Can't find file at `./no_such_file.csv`.",
-    fixed = TRUE
+    class = "frictionless_error_path_not_found"
   )
 
   # No file at paths
   p_invalid$resources[[1]]$path <- c("deployments.csv", "no_such_file.csv")
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Can't find file at `./deployments.csv`.",
-    fixed = TRUE
+    class = "frictionless_error_path_not_found"
   )
 
   # Path is absolute path
   p_invalid$resources[[1]]$path <- "/inst/extdata/deployments.csv"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    paste(
-      "`/inst/extdata/deployments.csv` is an absolute path (`/`)",
-      "which is unsafe."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_path_unsafe_absolute"
   )
 
   # Path is relative parent path
   p_invalid$resources[[1]]$path <- "../../inst/extdata/deployments.csv"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    paste(
-      "`../../inst/extdata/deployments.csv` is a relative parent path (`../`)",
-      "which is unsafe."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_path_unsafe_relative"
   )
 
   # Add valid path
@@ -100,9 +184,13 @@ test_that("read_resource() returns error on incorrect resource", {
   # Not a tabular-data-resource
   expect_error(
     read_resource(p_invalid, "deployments"),
-    paste(
-      "Resource `deployments` must have property `profile` with value",
-      "`tabular-data-resource`."
+    class = "frictionless_error_resource_not_tabular"
+  )
+  expect_error(
+    read_resource(p_invalid, "deployments"),
+    regexp = paste(
+      "Resource \"deployments\" must have a profile property with value",
+      "\"tabular-data-resource\"."
     ),
     fixed = TRUE
   )
@@ -111,7 +199,11 @@ test_that("read_resource() returns error on incorrect resource", {
   p_invalid$resources[[1]]$profile <- "tabular-data-resource"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Resource `deployments` must have property `schema`.",
+    class = "frictionless_error_resource_without_schema"
+  )
+  expect_error(
+    read_resource(p_invalid, "deployments"),
+    regexp = "Resource \"deployments\" must have a schema property.",
     fixed = TRUE
   )
 
@@ -119,24 +211,21 @@ test_that("read_resource() returns error on incorrect resource", {
   p_invalid$resources[[1]]$schema <- "http://example.com/no_schema.json"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Can't find file at `http://example.com/no_schema.json`.",
-    fixed = TRUE
+    class = "frictionless_error_url_not_found"
   )
 
   # No file at schema
   p_invalid$resources[[1]]$schema <- "no_schema.json"
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "Can't find file at `.*no_schema.json`",
-    # no fixed = TRUE, since full returned path depends on system
+    class = "frictionless_error_path_not_found"
   )
 
   # No fields
   p_invalid$resources[[1]]$schema <- list()
   expect_error(
     read_resource(p_invalid, "deployments"),
-    "`schema` must be a list with property `fields`.",
-    fixed = TRUE
+    class = "frictionless_error_schema_invalid"
   )
 
   # No field name
@@ -147,12 +236,7 @@ test_that("read_resource() returns error on incorrect resource", {
   )
   expect_error(
     read_resource(p_invalid, "deployments"),
-    paste(
-      "All fields in `schema` must have property `name`.",
-      "ℹ Field(s) `2` don't have a name.",
-      sep = "\n"
-    ),
-    fixed = TRUE
+    class = "frictionless_error_fields_without_name"
   )
 })
 
@@ -174,24 +258,24 @@ test_that("read_resource() can read inline data (ignoring schema)", {
   p$resources[[3]]$data <- "not_a_list" # Media resource
   expect_error(
     read_resource(p, "media"),
-    "second argument must be a list",
+    regexp = "second argument must be a list",
     fixed = TRUE
   )
 })
 
 test_that("read_resource() can read local files", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments") # local resource, remote package
 
-  p_local <- suppressMessages(read_package(
+  p_local <- read_package(
     system.file("extdata", "datapackage.json", package = "frictionless")
-  ))
+  )
   expect_identical(read_resource(p_local, "deployments"), resource)
 })
 
 test_that("read_resource() can read remote files", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments") # local resource, remote package
 
@@ -205,7 +289,7 @@ test_that("read_resource() can read remote files", {
 
 test_that("read_resource() can read safe local and remote Table Schema,
            including YAML", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments")
   p$directory <- "."
@@ -221,22 +305,14 @@ test_that("read_resource() can read safe local and remote Table Schema,
     "/tests/testthat/data/deployments_schema.json"
   expect_error(
     read_resource(p_unsafe, "deployments"),
-    paste(
-      "`/tests/testthat/data/deployments_schema.json` is an absolute path",
-      "(`/`) which is unsafe."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_path_unsafe_absolute"
   )
 
   # Schema is relative parent path
   p_unsafe$resources[[1]]$schema <- "../testthat/data/deployments_schema.json"
   expect_error(
     read_resource(p_unsafe, "deployments"),
-    paste(
-      "`../testthat/data/deployments_schema.json` is a relative parent path",
-      "(`../`) which is unsafe."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_path_unsafe_relative"
   )
 
   # Schema is local path
@@ -260,7 +336,7 @@ test_that("read_resource() can read safe local and remote Table Schema,
 })
 
 test_that("read_resource() can read safe local and remote CSV dialect", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments")
   p$directory <- "."
@@ -275,22 +351,14 @@ test_that("read_resource() can read safe local and remote CSV dialect", {
   p_unsafe$resources[[1]]$dialect <- "/tests/testthat/data/dialect.json"
   expect_error(
     read_resource(p_unsafe, "deployments"),
-    paste(
-      "`/tests/testthat/data/dialect.json` is an absolute path",
-      "(`/`) which is unsafe."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_path_unsafe_absolute"
   )
 
   # Dialect is relative parent path
   p_unsafe$resources[[1]]$dialect <- "../testthat/data/dialect.json"
   expect_error(
     read_resource(p_unsafe, "deployments"),
-    paste(
-      "`../testthat/data/dialect.json` is a relative parent path",
-      "(`../`) which is unsafe."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_path_unsafe_relative"
   )
 
   # Dialect is local path
@@ -313,7 +381,7 @@ test_that("read_resource() can read safe local and remote CSV dialect", {
 })
 
 test_that("read_resource() understands CSV dialect", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments")
 
@@ -343,7 +411,7 @@ test_that("read_resource() understands CSV dialect", {
 })
 
 test_that("read_resource() understands missing values", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments")
 
@@ -358,7 +426,7 @@ test_that("read_resource() understands missing values", {
 })
 
 test_that("read_resource() understands encoding", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments")
 
@@ -372,10 +440,14 @@ test_that("read_resource() understands encoding", {
   # Create package with unknown encoding
   p_unknown <- p
   p_unknown$resources[[1]]$encoding <- "utf-8-sig"
-  warnings <- capture_warnings(read_resource(p_unknown, "deployments"))
-  expect_identical(
-    warnings[1],
-    "Unknown encoding `utf-8-sig`. Reading file(s) with UTF-8 encoding."
+  expect_warning(
+    read_resource(p_unknown, "deployments"),
+    class = "frictionless_warning_resource_encoding_unknown"
+  )
+  expect_warning(
+    read_resource(p_unknown, "deployments"),
+    regexp = "Unknown encoding utf-8-sig. Reading file(s) with UTF-8 encoding.",
+    fixed = TRUE
   )
   expect_identical(
     suppressWarnings(read_resource(p_unknown, "deployments")),
@@ -385,7 +457,7 @@ test_that("read_resource() understands encoding", {
 
 test_that("read_resource() handles decimalChar/groupChar properties", {
   expected_value <- 3000000.3
-  p <- suppressMessages(read_package(test_path("data/mark.json")))
+  p <- read_package(test_path("data/mark.json"))
 
   # Default decimalChar/groupChar
   resource <- read_resource(p, "mark_default")
@@ -393,37 +465,41 @@ test_that("read_resource() handles decimalChar/groupChar properties", {
   expect_identical(resource$num_undefined, expected_value) # 3000000.30
 
   # Non-default decimalChar, default groupChar (which should not conflict)
-  warnings <- capture_warnings(read_resource(p, "mark_decimal"))
-  expect_identical(
-    warnings[1],
-    paste(
-      "Some fields define a non-default `decimalChar`.",
-      "Parsing all number fields with `,` as decimal mark."
-    )
+  expect_warning(
+    read_resource(p, "mark_decimal"),
+    class = "frictionless_warning_fields_decimalchar_different"
+  )
+  expect_warning(
+    read_resource(p, "mark_decimal"),
+    regexp = paste(
+      "Some fields define a non-default decimalChar.",
+      "Parsing all number fields with \",\" as decimal mark."
+    ),
+    fixed = TRUE
   )
 
   resource <- suppressWarnings(read_resource(p, "mark_decimal"))
   expect_identical(resource$num, expected_value) # 3000000.30
   expect_identical(resource$num_undefined, expected_value) # 3000000.30
 
-  # Non-default decimalChar/groupChar
-  warnings <- capture_warnings(read_resource(p, "mark_decimal_group"))
-  expect_true(length(warnings) == 3) # 2 warnings + 1 parsing failure last field
-  expect_identical(
-    warnings[1],
-    paste(
-      "Some fields define a non-default `decimalChar`.",
-      "Parsing all number fields with `,` as decimal mark."
-    )
-  )
-  expect_identical(
-    warnings[2],
-    paste(
-      "Some fields define a non-default `groupChar`.",
-      "Parsing all number fields with `.` as grouping mark."
-    )
-  )
-
+  # Non-default decimalChar and groupChar
+  # Results in 3 warnings: decimalchar, groupchar, parsing failure last field
+  suppressWarnings(expect_warning(
+    read_resource(p, "mark_decimal_group"),
+    class = "frictionless_warning_fields_decimalchar_different"
+  ))
+  suppressWarnings(expect_warning(
+    read_resource(p, "mark_decimal_group"),
+    class = "frictionless_warning_fields_groupchar_different"
+  ))
+  suppressWarnings(expect_warning(
+    read_resource(p, "mark_decimal_group"),
+    regexp = paste(
+      "Some fields define a non-default groupChar.",
+      "Parsing all number fields with \".\" as grouping mark."
+    ),
+    fixed = TRUE
+  ))
   resource <- suppressWarnings(read_resource(p, "mark_decimal_group"))
   expect_identical(resource$num, expected_value) # 3.000.000,30
   # Field without decimalChar is still parsed with non-default decimalChar
@@ -447,7 +523,7 @@ test_that("read_resource() handles LF and CRLF line terminator characters", {
   #
   # read_delim() however only handles 2 line terminator characters (LF and CRLF)
   # without explicitly indicating them, so dialect$lineTerminator is ignored
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments") # This file has LF
 
@@ -459,7 +535,7 @@ test_that("read_resource() handles LF and CRLF line terminator characters", {
 })
 
 test_that("read_resource() can read compressed files", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   resource <- read_resource(p, "deployments")
 
@@ -490,7 +566,7 @@ test_that("read_resource() can read compressed files", {
   # https://github.com/tidyverse/readr/issues/1042#issuecomment-545103047
   expect_error(
     read_resource(p_remote_zip, "deployments"),
-    paste(
+    regexp = paste(
       "Reading from remote `zip` compressed files is not supported,",
       "  download the files locally first.",
       sep = "\n"
@@ -502,7 +578,7 @@ test_that("read_resource() can read compressed files", {
 })
 
 test_that("read_resource() handles strings", {
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "string")
   expect_type(resource$str, "character")
 
@@ -513,7 +589,7 @@ test_that("read_resource() handles strings", {
 })
 
 test_that("read_resource() handles numbers", {
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "number")
 
   # Leading/trailing zeros are optional, + is assumed
@@ -546,7 +622,7 @@ test_that("read_resource() handles numbers", {
 })
 
 test_that("read_resource() handles integers (as doubles)", {
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "integer")
 
   # Leading/trailing zeros are optional, + is assumed
@@ -568,7 +644,7 @@ test_that("read_resource() handles integers (as doubles)", {
 })
 
 test_that("read_resource() handles booleans", {
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "boolean")
 
   # Default trueValues/falseValues are cast to logical
@@ -580,7 +656,7 @@ test_that("read_resource() handles booleans", {
 
 test_that("read_resource() handles dates", {
   expected_value <- as.Date("2013-11-23")
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "date")
   # This test covers:
   # - year: %Y %y
@@ -601,7 +677,7 @@ test_that("read_resource() handles dates", {
 
 test_that("read_resource() handles times", {
   expected_value <- hms::hms(0, 30, 8) # "08:30:00"
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "time")
   # This test covers:
   # - hour: %H (including 1 digit) %I + %p
@@ -625,7 +701,7 @@ test_that("read_resource() handles times", {
 
 test_that("read_resource() handles datetimes", {
   expected_value <- as.POSIXct("2013-11-23 08:30:00", tz = "UTC")
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "datetime")
 
   expect_identical(resource$dttm_undefined, resource$dttm_default)
@@ -640,7 +716,7 @@ test_that("read_resource() handles datetimes", {
 })
 
 test_that("read_resource() handles other types", {
-  p <- suppressMessages(read_package(test_path("data/types.json")))
+  p <- read_package(test_path("data/types.json"))
   resource <- read_resource(p, "other")
 
   # Interpret year, yearmonth as dates
