@@ -1,13 +1,11 @@
 test_that("write_package() returns output Data Package (invisibly)", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   # Note write_package() is expected to create directory without warning
   dir <- file.path(tempdir(), "package")
   on.exit(unlink(dir, recursive = TRUE))
   p_written <- suppressMessages(write_package(p, dir))
-  p_from_file <- suppressMessages(read_package(
-    file.path(dir, "datapackage.json")
-  ))
+  p_from_file <- read_package(file.path(dir, "datapackage.json"))
   # p_from_file$directory will differ: overwrite to make the same
   p_from_file$directory <- p_written$directory
 
@@ -15,14 +13,10 @@ test_that("write_package() returns output Data Package (invisibly)", {
   expect_identical(p_written, p_from_file)
 })
 
-test_that("write_package() returns error on incorrect Data Package", {
+test_that("write_package() returns error on invalid Data Package", {
   expect_error(
     write_package(list()),
-    paste(
-      "`package` must be a list describing a Data Package,",
-      "created with `read_package()` or `create_package()`."
-    ),
-    fixed = TRUE
+    class = "frictionless_error_package_invalid"
   )
 })
 
@@ -32,7 +26,16 @@ test_that("write_package() returns error if Data Package has no resource(s)", {
   on.exit(unlink(dir, recursive = TRUE))
   expect_error(
     write_package(p_empty, dir),
-    "`package` must have resources. Use `add_resource()` to add resources.",
+    class = "frictionless_error_package_without_resources"
+  )
+  expect_error(
+    write_package(p_empty, dir),
+    regexp = "`package` must have resources.",
+    fixed = TRUE
+  )
+  expect_error(
+    write_package(p_empty, dir),
+    regexp = "Use `add_resource()` to add resources.",
     fixed = TRUE
   )
 
@@ -41,11 +44,11 @@ test_that("write_package() returns error if Data Package has no resource(s)", {
 })
 
 test_that("write_package() writes unaltered datapackage.json as is", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p_file <- system.file("extdata", "datapackage.json", package = "frictionless")
   json_original <- readr::read_lines(p_file) # Will use line endings of system
 
-  p <- suppressMessages(read_package(p_file))
+  p <- read_package(p_file)
   dir <- file.path(tempdir(), "package")
   on.exit(unlink(dir, recursive = TRUE))
   suppressMessages(write_package(p, dir))
@@ -57,10 +60,10 @@ test_that("write_package() writes unaltered datapackage.json as is", {
 })
 
 test_that("write_package() does not overwrite existing data files", {
-  testthat::skip_if_offline()
-  p <- suppressMessages(read_package(
+  skip_if_offline()
+  p <- read_package(
     system.file("extdata", "datapackage.json", package = "frictionless")
-  ))
+  )
   dir <- file.path(tempdir(), "package")
   on.exit(unlink(dir, recursive = TRUE))
   dir.create(dir)
@@ -68,25 +71,22 @@ test_that("write_package() does not overwrite existing data files", {
   # Create files in directory
   files <- c(
     datapackage = file.path(dir, "datapackage.json"),
-    deployments = file.path(dir, "deployments.csv"),
-    observations_1 = file.path(dir, "observations_1.csv"),
-    observations_2 = file.path(dir, "observations_2.csv")
+    deployments = file.path(dir, "deployments.csv") # Local path
+    # observations_1, observations_2 have remote path and would not be written
   )
   file.create(files) # Size for these files will be 0
 
   # Write package to directory, expect only datapackage.json is overwritten
   suppressMessages(write_package(p, dir))
   expect_gt(file.info(files["datapackage"])$size, 0) # Overwitten
-  expect_equal(file.info(files["deployments"])$size, 0)
-  expect_equal(file.info(files["observations_1"])$size, 0)
-  expect_equal(file.info(files["observations_2"])$size, 0)
+  expect_identical(file.info(files["deployments"])$size, 0) # Remains the same
 })
 
 test_that("write_package() copies file(s) for path = local in local package", {
-  testthat::skip_if_offline()
-  p <- suppressMessages(read_package(
+  skip_if_offline()
+  p <- read_package(
     system.file("extdata", "datapackage.json", package = "frictionless")
-  ))
+  )
   p$resources[[2]]$path[[2]] <- "observations_2.csv" # Make one URL a local path
   p <- add_resource(p, "new", test_path("data/df.csv"))
   dir <- file.path(tempdir(), "package")
@@ -112,7 +112,7 @@ test_that("write_package() copies file(s) for path = local in local package", {
 
 test_that("write_package() downloads file(s) for path = local in remote
            package", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   p$resources[[2]]$path[[2]] <- "observations_2.csv" # Make one URL a local path
   p <- add_resource(p, "new", test_path("data/df.csv"))
@@ -138,10 +138,10 @@ test_that("write_package() downloads file(s) for path = local in remote
 })
 
 test_that("write_package() leaves as is for path = URL in local package", {
-  testthat::skip_if_offline()
-  p <- suppressMessages(read_package(
+  skip_if_offline()
+  p <- read_package(
     system.file("extdata", "datapackage.json", package = "frictionless")
-  ))
+  )
   p <- add_resource(p, "new", file.path(
     "https://raw.githubusercontent.com/frictionlessdata/frictionless-r",
     "main/tests/testthat/data/df.csv"
@@ -161,7 +161,7 @@ test_that("write_package() leaves as is for path = URL in local package", {
 })
 
 test_that("write_package() leaves as is for path = URL in remote package", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   p <- add_resource(p, "new", file.path(
     "https://raw.githubusercontent.com/frictionlessdata/frictionless-r",
@@ -182,10 +182,10 @@ test_that("write_package() leaves as is for path = URL in remote package", {
 })
 
 test_that("write_package() leaves as is for data = json in local package", {
-  testthat::skip_if_offline()
-  p <- suppressMessages(read_package(
+  skip_if_offline()
+  p <- read_package(
     system.file("extdata", "datapackage.json", package = "frictionless")
-  ))
+  )
   dir <- file.path(tempdir(), "package")
   on.exit(unlink(dir, recursive = TRUE))
   p_written <- suppressMessages(write_package(p, dir))
@@ -198,7 +198,7 @@ test_that("write_package() leaves as is for data = json in local package", {
 })
 
 test_that("write_package() leaves as is for data = json in remote package", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   dir <- file.path(tempdir(), "package")
   on.exit(unlink(dir, recursive = TRUE))
@@ -212,10 +212,10 @@ test_that("write_package() leaves as is for data = json in remote package", {
 })
 
 test_that("write_package() creates file for data = df in local package", {
-  testthat::skip_if_offline()
-  p <- suppressMessages(read_package(
+  skip_if_offline()
+  p <- read_package(
     system.file("extdata", "datapackage.json", package = "frictionless")
-  ))
+  )
   df <- data.frame("col_1" = c(1, 2), "col_2" = c("a", "b"))
   p <- add_resource(p, "new", df)
   dir <- file.path(tempdir(), "package")
@@ -231,7 +231,7 @@ test_that("write_package() creates file for data = df in local package", {
 })
 
 test_that("write_package() creates file for data = df in remote package", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   df <- data.frame("col_1" = c(1, 2), "col_2" = c("a", "b"))
   p <- add_resource(p, "new", df)
@@ -248,22 +248,28 @@ test_that("write_package() creates file for data = df in remote package", {
 })
 
 test_that("write_package() shows message when downloading file", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   dir <- file.path(tempdir(), "package")
+  dir_1 <- file.path(dir, "1")
+  dir_2 <- file.path(dir, "2")
   on.exit(unlink(dir, recursive = TRUE))
   expect_message(
-    write_package(p, dir),
-    paste0(
-      "Downloading file from https://raw.githubusercontent.com/",
-      "frictionlessdata/frictionless-r/main/inst/extdata/deployments.csv"
+    write_package(p, dir_1),
+    class = "frictionless_message_file_downloading"
+  )
+  expect_message(
+    write_package(p, dir_2),
+    regexp = paste0(
+      "Downloading file from 'https://raw.githubusercontent.com/",
+      "frictionlessdata/frictionless-r/main/inst/extdata/deployments.csv'"
     ),
     fixed = TRUE
   )
 })
 
 test_that("write_package() sets correct properties for data frame resources", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   df <- data.frame("col_1" = c(1, 2), "col_2" = c("a", "b"))
   schema <- create_schema(df)
@@ -277,23 +283,33 @@ test_that("write_package() sets correct properties for data frame resources", {
   expect_identical(resource_written$name, "new")
   expect_identical(resource_written$path, "new.csv")
   expect_identical(resource_written$profile, "tabular-data-resource")
-  expect_null(resource_written$title)
-  expect_null(resource_written$description)
   expect_identical(resource_written$format, "csv")
   expect_identical(resource_written$mediatype, "text/csv")
   expect_identical(resource_written$encoding, "utf-8")
   expect_null(resource_written$dialect)
-  expect_null(resource_written$bytes)
-  expect_null(resource_written$hash)
-  expect_null(resource_written$sources)
-  expect_null(resource_written$licenses)
   expect_identical(resource_written$schema, schema)
   expect_null(resource_written$data)
   expect_null(resource_written$read_from)
 })
 
+test_that("write_package() retains custom properties set in add_resource()", {
+  p <- create_package()
+  df <- data.frame("col_1" = c(1, 2), "col_2" = c("a", "b"))
+  p <- add_resource(p, "new_df", df, title = "custom_title", foo = "bar")
+  df_csv <- test_path("data/df.csv")
+  p <- add_resource(p, "new_csv", df_csv, title = "custom_title", foo = "bar")
+  dir <- file.path(tempdir(), "package")
+  on.exit(unlink(dir, recursive = TRUE))
+  p_written <- suppressMessages(write_package(p, dir))
+
+  expect_identical(p_written$resources[[1]]$title, "custom_title")
+  expect_identical(p_written$resources[[1]]$foo, "bar")
+  expect_identical(p_written$resources[[2]]$title, "custom_title")
+  expect_identical(p_written$resources[[2]]$foo, "bar")
+})
+
 test_that("write_package() will gzip file for compress = TRUE", {
-  testthat::skip_if_offline()
+  skip_if_offline()
   p <- example_package
   df <- data.frame("col_1" = c(1, 2), "col_2" = c("a", "b"))
   p <- add_resource(p, "new", df)
@@ -308,6 +324,6 @@ test_that("write_package() will gzip file for compress = TRUE", {
   expect_false(file.exists(file.path(dir, "new.csv")))
 
   # Written file can be read by read_resource()
-  p_reread <- suppressMessages(read_package(file.path(dir, "datapackage.json")))
+  p_reread <- read_package(file.path(dir, "datapackage.json"))
   expect_identical(read_resource(p_reread, "new"), dplyr::as_tibble(df))
 })
